@@ -2,11 +2,11 @@ from modeller import *
 from modeller.automodel import *
 
 log.verbose()
-env = environ()
+env = Environ()
 
 env.io.atom_files_directory = ['.', '../atom_files']
 
-a = allhmodel(env, alnfile='alignment.ali', knowns='5fd1', sequence='1fdx')
+a = AllHModel(env, alnfile='alignment.ali', knowns='5fd1', sequence='1fdx')
 a.starting_model = a.ending_model = 4
 
 a.make()
