@@ -7,14 +7,14 @@ from modeller import *
 from modeller.scripts import complete_pdb
 
 log.verbose()
-env = environ()
+env = Environ()
 env.io.atom_files_directory = ['../atom_files']
 env.libs.topology.read(file='$(LIB)/top_heav.lib')
 env.libs.parameters.read(file='$(LIB)/par.lib')
 
 mdl = complete_pdb(env, '1fas')
 
-allsel = selection(mdl)
+allsel = Selection(mdl)
 mdl.restraints.make(allsel, restraint_type='stereo', spline_on_site=False)
 allsel.energy()
 
