@@ -2,6 +2,7 @@
 
 from modeller.util.modlist import FixList
 
+
 def __smooth_one(vals, n, window):
     """Return the weighted average of ``window`` values around ``vals[n]``"""
     i1 = max(0, n - window)
@@ -13,10 +14,12 @@ def __smooth_one(vals, n, window):
         wtsum += weight
     return av / max(wtsum, 0.1)
 
+
 def _smooth(vals, window):
     """Return a smoothed copy of the array ``vals``"""
     return [__smooth_one(vals, n, window) for n in range(len(vals))]
 
+
 class EnergyProfileResidue(object):
     """A single residue in the energy profile."""
     def __init__(self, energy, num_restraints):
@@ -36,11 +39,12 @@ class EnergyProfileResidue(object):
                                      " is divided by the number of"
                                      " restraints acting on this residue")
 
+
 class EnergyProfile(FixList):
     """A per-residue energy profile"""
 
     def __init__(self, profile, nprofile, min_rms, heavy_rms):
-        self.__profile = [EnergyProfileResidue(energy, num_restraints) \
+        self.__profile = [EnergyProfileResidue(energy, num_restraints)
                           for energy, num_restraints in zip(profile, nprofile)]
         self.min_rms = min_rms
         self.heavy_rms = heavy_rms
