@@ -3,24 +3,36 @@
 # version check
 
 import modeller.features
+import modeller.forms
+import _modeller
+import string
+
+
+class _AsymIDs(object):
+    """Map indices to multi-character asym (chain) IDs.
+       We label the first 26 chains A-Z, then we move to two-letter
+       chain IDs: AA through AZ, then BA through BZ, through to ZZ.
+       This continues with longer chain IDs."""
+    def __getitem__(self, ind):
+        chars = string.ascii_uppercase
+        lc = len(chars)
+        ids = []
+        while ind >= lc:
+            ids.append(chars[ind % lc])
+            ind = ind // lc - 1
+        ids.append(chars[ind])
+        return "".join(reversed(ids))
 
 
 class _CifAtom(object):
     """Map a Modeller Atom to mmCIF ids"""
-    def __init__(self, atom, entity_map):
+    def __init__(self, atom, asym_map):
         self._atom = atom
-        chain = atom.residue.chain
-        # Modeller chain index is 1-based
-        self.entity_id = entity_map[chain.index - 1]
-        # Get seq_id offset (index is from start of model, but we need it to
-        # be from the start of the chain)
-        self.chain_offset = chain.residues[0].index - 1
+        self.asym_id, self.entity_id, self.seq_id = asym_map.map_residue(
+            atom.residue)
     # Evaluate only when needed
     atom_id = property(lambda self: self._atom.name)
     comp_id = property(lambda self: self._atom.residue.name)
-    asym_id = property(lambda self: self._atom.residue.chain.name)
-    seq_id = property(
-        lambda self: self._atom.residue.index - self.chain_offset)
 
 
 class TemplateChain(object):
@@ -44,11 +56,6 @@ class Data(object):
     pass
 
 
-class DataGroup(list):
-    """A group of data."""
-    pass
-
-
 class TemplateData(Data):
     """Template data used in the modeling."""
     content_type = 'template structure'
@@ -61,7 +68,7 @@ class AlignmentData(Data):
 
 class RestraintData(Data):
     """Restraint data used in the modeling."""
-    content_type = 'restraint'
+    content_type = 'spatial restraints'
 
 
 class TargetData(Data):
@@ -76,62 +83,113 @@ class ModelData(Data):
 
 class Restraints(object):
     """A set of restraints used in the modeling."""
-    pass
+    def _get_atom_keywords(self):
+        """Get the mmCIF keywords for the atoms in this restraint type"""
+        kws = []
+        for i in range(1, self._num_atoms + 1):
+            kws.extend(('entity_id_%d' % i, 'asym_id_%d' % i, 'seq_id_%d' % i,
+                        'comp_id_%d' % i, 'atom_id_%d' % i))
+        return kws
+
+    def _get_granularity_keyword(self):
+        # Only distance restraints have a granularity in the dictionary
+        return []
 
 
 class DistanceRestraints(Restraints, list):
     """A list of distance restraints used in the modeling."""
-    restraint_type = 'Distance restraints'
+    _num_atoms = 2
+    restraint_type = 'distance restraint'
+    category = '_ma_distance_restraints'
+
+    def _get_granularity_keyword(self):
+        return ['granularity']
+
+
+class AngleRestraints(Restraints, list):
+    """A list of angle restraints used in the modeling."""
+    _num_atoms = 3
+    restraint_type = 'angle restraint'
+    category = '_ma_angle_restraints'
+
+
+class DihedralRestraints(Restraints, list):
+    """A list of dihedral restraints used in the modeling."""
+    _num_atoms = 4
+    restraint_type = 'dihedral restraint'
+    category = '_ma_dihedral_restraints'
+
+
+class _AsymMap(object):
+    def __init__(self, mdl):
+        self.mdl = mdl
+        self._asym_from_int_id = _AsymIDs()
+        _modeller.mod_model_make_mmcif_asyms(mdl.modpt, mdl.env.libs.modpt)
+
+    def map_residue(self, res):
+        """Map a Residue object to asym, entity, sequence IDs"""
+        int_asym_id, entity_id, seq_id = _modeller.mod_model_get_residue_asym(
+            self.mdl.modpt, res._num)
+        return self._asym_from_int_id[int_asym_id], entity_id, seq_id
+
+    def map_chain(self, nchain):
+        """Map a chain index to polymer asym, entity IDs.
+           If the given chain does not contain a polymer, return None for
+           each."""
+        int_asym_id, entity_id = _modeller.mod_model_get_chain_asym(
+            self.mdl.modpt, nchain)
+        if entity_id == 0:
+            return None, None
+        else:
+            return self._asym_from_int_id[int_asym_id], entity_id
 
 
 class CifData(object):
-    def __init__(self):
+    def __init__(self, mdl):
         self.data = []
-        self.data_groups = []
-        self.modeling_input_data = DataGroup()
-        self._add_data_group(self.modeling_input_data)
-        self.modeling_output_data = DataGroup()
-        self._add_data_group(self.modeling_output_data)
-
-    def add_alignment_info(self, aln, knowns, seq, mdl):
         self.mdl = mdl
-        self._add_target_chains(aln[seq], mdl)
+        _modeller.mod_model_clear_mmcif_file_data(self.mdl.modpt)
+        self._asym_map = _AsymMap(mdl)
+
+    def add_alignment_info(self, aln, knowns, seq):
+        self._add_target_chains(aln[seq], self.mdl)
         self._add_template_chains(aln, knowns, aln[seq])
         self._add_alignments(aln)
 
     def add_restraints(self, user_restraints):
         self.restraints = []
-        # Only handle single-feature restraints for now
-        self._add_distance_restraints(
-            r for r in user_restraints
-            if isinstance(r._features, modeller.features.Distance))
+        # Only handle single-feature Gaussian-type restraints for now
+        gauss_restraints = [r for r in user_restraints
+                            if isinstance(r, (modeller.forms.Gaussian,
+                                              modeller.forms.UpperBound,
+                                              modeller.forms.LowerBound))]
+        for rsrcls, feat in [
+                (DistanceRestraints, modeller.features.Distance),
+                (AngleRestraints, modeller.features.Angle),
+                (DihedralRestraints, modeller.features.Dihedral)]:
+            rsr = rsrcls(r for r in gauss_restraints
+                         if isinstance(r._features, feat))
+            if len(rsr) > 0:
+                self.restraints.append(rsr)
+                rsr.id = len(self.restraints)
 
-    def _add_distance_restraints(self, rs):
-        self.distance_restraints = DistanceRestraints(rs)
-        if len(self.distance_restraints) > 0:
-            self._add_restraint(self.distance_restraints)
+        if self.restraints:
+            # add Data for restraints
+            d = RestraintData()
+            d.name = 'User-provided restraints'
+            self._add_data(d, True)
+            for rs in self.restraints:
+                rs.data = d
 
-    def _add_data(self, data, data_group):
+    def _add_data(self, data, is_input):
         """Add some data used in the modeling."""
         self.data.append(data)
         data.id = len(self.data)
-        data_group.append(data)
-
-    def _add_data_group(self, data_group):
-        """Add some a group of data"""
-        self.data_groups.append(data_group)
-        data_group.id = len(self.data_groups)
-
-    def _add_restraint(self, rs):
-        """Add a set of restraints."""
-        self.restraints.append(rs)
-        rs.id = len(self.restraints)
-
-        # add Data for this set of restraints
-        d = RestraintData()
-        d.name = 'User-provided restraints'
-        rs.data = d
-        self._add_data(d, self.modeling_input_data)
+        # Register data type with the lower-level Fortran code, as that
+        # writes out the ma_data mmCIF table and other tables that need
+        # data IDs
+        _modeller.mod_model_add_mmcif_file_data(self.mdl.modpt, data.name,
+                                                data.content_type, is_input)
 
     def _get_gapped_sequence(self, chain):
         """Return a sequence including gaps for the given chain"""
@@ -146,17 +204,20 @@ class CifData(object):
         # add Data for this alignment
         d = AlignmentData()
         d.name = 'Target Template Alignment'
-        self._add_data(d, self.modeling_input_data)
+        self._add_data(d, True)
 
         target_to_alignment = {}
         self.alignments = []
-        for nchain, target in enumerate(self.target_chains):
+        for target in self.target_chains:
+            # Skip non-polymer chains
+            if target is None:
+                continue
             a = Alignment()
             target_to_alignment[target] = a
             a.data = d
             a.target = target
             a.templates = []
-            a.id = nchain + 1
+            a.id = target.id
             self.alignments.append(a)
         # Add template information
         for template in self.template_chains:
@@ -181,13 +242,16 @@ class CifData(object):
             # Add Data for this template
             d = TemplateData()
             d.name = 'Template Structure'
-            self._add_data(d, self.modeling_input_data)
+            self._add_data(d, True)
 
             seq = aln[k]
             for chain in seq.chains:
+                target_chain = self._get_target_chain(chain, target)
+                # If chain contains only ligands, we can't align it
+                if target_chain is None:
+                    continue
                 t = TemplateChain()
-                # Assume PDB code is first 4 characters of align code
-                t.pdb_code = k[:4].upper()
+                t.pdb_accession = seq.pdb_accession
                 t.id = ordinal
                 t.asym_id = chain.name
                 t.template_data = d
@@ -196,31 +260,25 @@ class CifData(object):
                 t.sequence = ''.join(r.code for r in chain.residues)
                 t.gapped_sequence = self._get_gapped_sequence(chain)
                 t.sequence_can = ''.join(r.code for r in chain.residues)
-                t.target_chain = self._get_target_chain(chain, target)
+                t.target_chain = target_chain
                 self.template_chains.append(t)
                 ordinal += 1
 
-    def _get_target_entities(self, seq):
-        # Mapping from chain # to entity #
-        self._entity_map = {}
-        seen_seqs = {}
-        for nchain, chain in enumerate(seq.chains):
-            s = tuple(r.type for r in chain.residues)
-            seen_seqs[s] = None
-            self._entity_map[nchain] = len(seen_seqs)
-
     def _add_target_chains(self, seq, mdl):
-        self._get_target_entities(seq)
         ordinal = 1
         self.target_chains = []
         for nchain, chain in enumerate(seq.chains):
-            model_chain = mdl.chains[nchain]
+            asym_id, entity_id = self._asym_map.map_chain(nchain)
+            # No polymer in this Modeller segment. Add a None chain so
+            # that we can still use the chain index to access the
+            # target_chains list.
+            if asym_id is None:
+                self.target_chains.append(None)
+                continue
             t = TargetChain()
             t.id = ordinal
-            # Use output model chain IDs, not the original sequence (chain
-            # IDs may have been changed)
-            t.asym_id = model_chain.name
-            t.entity_id = self._entity_map[nchain]
+            t.asym_id = asym_id
+            t.entity_id = entity_id
             t.sequence = ''.join(r.code for r in chain.residues)
             t.gapped_sequence = self._get_gapped_sequence(chain)
             t.seq_range = (1, len(chain.residues))
@@ -230,30 +288,15 @@ class CifData(object):
     def write_mmcif(self, writer):
         target_seq = TargetData()
         target_seq.name = 'Target Sequence'
-        self._add_data(target_seq, self.modeling_input_data)
+        self._add_data(target_seq, True)
         model_data = ModelData()
         model_data.name = 'Target Structure'
-        self._add_data(model_data, self.modeling_output_data)
-        self._write_software_groups(writer)
+        self._add_data(model_data, False)
         self._write_template_details(writer)
         self._write_template_segments(writer)
-        self._write_target_entity(writer, target_seq)
         self._write_poly_mapping(writer)
         self._write_alignment(writer)
-        self._write_data(writer)
-        self._write_data_groups(writer)
         self._write_restraints(writer)
-        self._write_distance_restraints(writer)
-        self._write_protocols(writer)
-        self._write_model_list(writer, model_data)
-
-    def _write_software_groups(self, writer):
-        # Write a single group that includes software #1 (Modeller, already
-        # written out by Model.write())
-        with writer.loop("_ma_software_group",
-                         ["ordinal_id", "group_id", "software_id",
-                          "parameter_group_id"]) as lp:
-            lp.write(ordinal_id=1, group_id=1, software_id=1)
 
     def _write_template_details(self, writer):
         with writer.loop(
@@ -278,14 +321,23 @@ class CifData(object):
                           'template_model_num',
                           'template_auth_asym_id']) as lp:
             for t in self.template_chains:
+                origin = 'reference database' if t.pdb_accession else '?'
                 lp.write(ordinal_id=t.id, template_id=t.id,
-                         template_origin='?',
+                         template_origin=origin,
                          template_entity_type='polymer',
                          template_trans_matrix_id=1,
                          template_data_id=t.template_data.id,
                          target_asym_id=t.target_chain.asym_id,
                          template_model_num=1, template_auth_asym_id=t.asym_id)
 
+        with writer.loop('_ma_template_ref_db_details',
+                         ['template_id', 'db_name',
+                          'db_accession_code']) as lp:
+            for t in self.template_chains:
+                if t.pdb_accession:
+                    lp.write(template_id=t.id, db_name='PDB',
+                             db_accession_code=t.pdb_accession)
+
         with writer.loop('_ma_template_poly',
                          ['template_id', 'seq_one_letter_code',
                           'seq_one_letter_code_can']) as lp:
@@ -294,7 +346,6 @@ class CifData(object):
                          seq_one_letter_code=t.sequence,
                          seq_one_letter_code_can=t.sequence_can)
 
-
     def _write_template_segments(self, writer):
         ordinal = 1
         with writer.loop('_ma_template_poly_segment',
@@ -307,20 +358,6 @@ class CifData(object):
                 t.segment_id = ordinal
                 ordinal += 1
 
-    def _write_target_entity(self, writer, target_seq):
-        all_entities = set(t.entity_id for t in self.target_chains)
-        with writer.loop('_ma_target_entity',
-                         ['entity_id', 'data_id', 'origin']) as lp:
-            for entity_id in sorted(all_entities):
-                lp.write(entity_id=entity_id,
-                         data_id=target_seq.id, origin='?')
-
-        with writer.loop('_ma_target_entity_instance',
-                         ['asym_id', 'entity_id', 'details']) as lp:
-            for t in self.target_chains:
-                lp.write(asym_id=t.asym_id, entity_id=t.entity_id,
-                         details="Model subunit")
-
     def _write_poly_mapping(self, writer):
         ordinal = 1
         with writer.loop('_ma_target_template_poly_mapping',
@@ -367,79 +404,66 @@ class CifData(object):
                          target_template_flag=1,  # Target
                          sequence=a.target.gapped_sequence)
 
-    def _write_data(self, writer):
-        with writer.loop('_ma_data',
-                         ['id', 'name', 'content_type']) as lp:
-            for d in self.data:
-                lp.write(id=d.id, name=d.name, content_type=d.content_type)
-
-    def _write_data_groups(self, writer):
-        ordinal = 1
-        with writer.loop('_ma_data_group',
-                         ['ordinal_id', 'group_id', 'data_id']) as lp:
-            for g in self.data_groups:
-                for d in g:
-                    lp.write(ordinal_id=ordinal, group_id=g.id, data_id=d.id)
-                    ordinal += 1
-
     def _write_restraints(self, writer):
         with writer.loop('_ma_restraints',
-                         ['ordinal_id', 'restraint_id', 'data_id', 'name',
+                         ['restraint_id', 'data_id', 'name',
                           'restraint_type', 'details']) as lp:
-            ordinal = 1
             for r in self.restraints:
-                lp.write(ordinal_id=ordinal, restraint_id=r.id,
-                         data_id=r.data.id, restraint_type=r.restraint_type)
-                ordinal += 1
+                lp.write(restraint_id=r.id, data_id=r.data.id,
+                         restraint_type=r.restraint_type)
+        for r in self.restraints:
+            self._write_type_restraints(writer, r)
 
-    def _write_distance_restraints(self, writer):
-        with writer.loop('_ma_distance_restraints',
-                         ['ordinal_id', 'restraint_id', 'group_id',
-                          'entity_id_1', 'asym_id_1', 'seq_id_1', 'comp_id_1',
-                          'atom_id_1', 'entity_id_2', 'asym_id_2', 'seq_id_2',
-                          'comp_id_2', 'atom_id_2',
-                          'distance_threshold', 'uncertainty']) as lp:
+    def _get_rsr_limits(self, r):
+        """Get the limits for a given restraint"""
+        mean, stddev = r._parameters[0:2]
+        lower = upper = lower_esd = upper_esd = '.'
+        if isinstance(r, modeller.forms.Gaussian):
+            rsr_type = 'lower and upper bound'
+            lower = upper = mean
+            lower_esd = upper_esd = stddev
+        elif isinstance(r, modeller.forms.LowerBound):
+            rsr_type = 'lower bound'
+            lower = mean
+            lower_esd = stddev
+        else:
+            rsr_type = 'upper bound'
+            upper = mean
+            upper_esd = stddev
+        return lower, upper, lower_esd, upper_esd, rsr_type
+
+    def _get_rsr_atom_items(self, r):
+        """Get the mmCIF data items for the atoms in a given restraint"""
+        atoms = r._features.indices_to_atoms(
+            self.mdl, r._features.get_atom_indices()[0])
+        atoms = [_CifAtom(a, self._asym_map) for a in atoms]
+        keys = {}
+        for i, atom in enumerate(atoms):
+            ki = "%d" % (i + 1)
+            keys['atom_id_' + ki] = atom.atom_id
+            keys['comp_id_' + ki] = atom.comp_id
+            keys['asym_id_' + ki] = atom.asym_id
+            keys['seq_id_' + ki] = atom.seq_id
+            keys['entity_id_' + ki] = atom.entity_id
+        return keys
+
+    def _write_type_restraints(self, writer, restraints):
+        """Write one type of restraint (distance, angle, dihedral)"""
+        with writer.loop(restraints.category,
+                         ['ordinal_id', 'restraint_id', 'restraint_type']
+                         + restraints._get_atom_keywords()
+                         + restraints._get_granularity_keyword()
+                         + ['lower_limit', 'lower_limit_esd',
+                            'upper_limit', 'upper_limit_esd']) as lp:
             ordinal = 1
-            for r in self.distance_restraints:
-                # Assume all such restraints are Gaussian for now
-                atoms = r._features.indices_to_atoms(
-                    self.mdl, r._features.get_atom_indices()[0])
-                atoms = [_CifAtom(a, self._entity_map) for a in atoms]
+            for r in restraints:
+                (lower, upper, lower_esd,
+                 upper_esd, rsr_type) = self._get_rsr_limits(r)
+                atom_items = self._get_rsr_atom_items(r)
                 lp.write(ordinal_id=ordinal,
-                         restraint_id=self.distance_restraints.id,
-                         group_id=1,
-                         atom_id_1=atoms[0].atom_id,
-                         comp_id_1=atoms[0].comp_id,
-                         asym_id_1=atoms[0].asym_id,
-                         seq_id_1=atoms[0].seq_id,
-                         entity_id_1=atoms[0].entity_id,
-                         atom_id_2=atoms[1].atom_id,
-                         comp_id_2=atoms[1].comp_id,
-                         asym_id_2=atoms[1].asym_id,
-                         seq_id_2=atoms[1].seq_id,
-                         entity_id_2=atoms[1].entity_id,
-                         distance_threshold=r._parameters[0],
-                         uncertainty=r._parameters[1])
+                         restraint_id=restraints.id,
+                         granularity='by-atom', restraint_type=rsr_type,
+                         upper_limit=upper, lower_limit=lower,
+                         upper_limit_esd=upper_esd, lower_limit_esd=lower_esd,
+                         **atom_items)
                 ordinal += 1
-
-    def _write_protocols(self, writer):
-        # We write a single 'modeling' step using software group 1 (Modeller)
-        with writer.loop('_ma_protocol_step',
-                         ['ordinal_id', 'protocol_id', 'step_id',
-                          'method_type', 'step_name', 'details',
-                          'software_group_id', 'input_data_group_id',
-                          'output_data_group_id']) as lp:
-            lp.write(ordinal_id=1, protocol_id=1, step_id=1,
-                     method_type='modeling', software_group_id=1,
-                     input_data_group_id=self.modeling_input_data.id,
-                     output_data_group_id=self.modeling_output_data.id)
-
-    def _write_model_list(self, writer, model_data):
-        with writer.loop('_ma_model_list',
-                         ['ordinal_id', 'model_id', 'model_group_id',
-                          'model_name', 'model_group_name', 'data_id',
-                          'model_type', 'model_type_other_details']) as lp:
-            lp.write(ordinal_id=1, model_id=1, model_group_id=1,
-                     model_name="Target Structure",
-                     model_group_name="All models",
-                     data_id=model_data.id, model_type="Homology model")
