@@ -3,6 +3,7 @@
 import _modeller
 from modeller import modfile
 
+
 class Topology(object):
     """All information from a residue topology file. You should never need to
        create a :class:`Topology` object yourself - one is created for you
@@ -34,7 +35,8 @@ class Topology(object):
                   sub-topology model as defined in ``models.lib``.
         """
         self.submodel = submodel
-        return _modeller.mod_topology_model_make(self._modpt, self.__libs.modpt)
+        return _modeller.mod_topology_model_make(self._modpt,
+                                                 self.__libs.modpt)
 
     def write(self, file):
         """Writes topology library to a file"""
@@ -45,13 +47,15 @@ class Topology(object):
 
     def __get_submodel(self):
         return _modeller.mod_topology_submodel_get(self._modpt)
+
     def __set_submodel(self, val):
         _modeller.mod_topology_submodel_set(self._modpt, val)
+
     def __get_in_memory(self):
         return _modeller.mod_topology_in_memory(self._modpt)
 
     submodel = property(__get_submodel, __set_submodel,
-                        doc="Topology sub-model, an integer from 1 to 10, " + \
+                        doc="Topology sub-model, an integer from 1 to 10, "
                             "as defined in ``models.lib``")
     in_memory = property(__get_in_memory,
                          doc="True if information has been read into memory")
